-- ErrConv 1.40
-- Create 2 .reg files, for associating and unassociating a file type
-- and corresponding extensions with an icon and a program on Windows.
--
-- Public Domain -- 2007, Dec 18
-- by Juergen Luethje <support {AT} luethje {DOT} eu>
-- Standard disclaimer: Use at your own risk!

include get.e                              -- for wait_key()
include wildcard.e                         -- for lower()

--------------------------[ utility routines ]--------------------------

global procedure wait (sequence_of_byte msg)
   integer void

   puts(2, msg & "\n\nPress any key ...")
   void = wait_key()
   puts(2, "\n")
end procedure

--------------------------------[ main ]--------------------------------

function reg_escape (sequence_of_byte fileName)
   sequence ret
   integer prevChar, char

   ret = ""
   prevChar = -1
   for i = 1 to length(fileName) do
      char = fileName[i]
      if char = '\\' and  prevChar != '\\'
      and (i = length(fileName) or find(fileName[i+1], "\\\"") = 0) then
         ret &= "\\\\"
      else
         ret &= char
      end if
      prevChar = char
   end for
   return ret
end function

function open_regfile (sequence_of_byte regFile)
   integer fw

   if not equal(lower(regFile[$-3..$]), ".reg") then
      regFile &= ".reg"
   end if

   fw = open(regFile, "w")
   if fw = -1 then
      return -1                            -- error
   end if

   puts(fw, "REGEDIT4\n\n")
   return fw
end function

procedure close_regfile (integer file)
   puts(file, "\n")
   close(file)
end procedure

function reg_associate (
                        sequence_of_byte regFile,
                        sequence extensionList,
                        sequence_of_byte fileType,
                        sequence_of_byte description,
                        sequence_of_byte iconFile, integer iconNo,
                        sequence_of_byte progFile
                       )
   -- Creates the file <regFile>.
   -- When then <regFile> is double-clicked, files with an
   -- extension contained in <extensionList> will be associated with
   -- the denoted icon and with program <progFile>.
   integer fw

   fw = open_regfile(regFile)
   if fw = -1 then
      return -1                            -- error
   end if

   for i = 1 to length(extensionList) do
      printf(fw, "[HKEY_CLASSES_ROOT\\.%s]\n", {extensionList[i]})
      printf(fw, "@=\"%s\"\n\n", {fileType})
   end for

   printf(fw, "[HKEY_CLASSES_ROOT\\%s]\n", {fileType})
   printf(fw, "@=\"%s\"\n\n", {description})

   printf(fw, "[HKEY_CLASSES_ROOT\\%s\\DefaultIcon]\n", {fileType})
   printf(fw, "@=\"\\\"%s\\\",%d\"\n\n", {reg_escape(iconFile), iconNo})

   printf(fw, "[HKEY_CLASSES_ROOT\\%s\\Shell\\Open\\Command]\n", {fileType})
   printf(fw, "@=\"\\\"%s\\\" \\\"%%1\\\"\"", {reg_escape(progFile)})

   close_regfile(fw)
   return 0                                -- success
end function

function reg_unassociate (
                          sequence_of_byte regFile,
                          sequence extensionList,
                          sequence_of_byte fileType
                         )
   -- According to www.wotsit.org/search.asp?page=3&s=windows:
   -- Delete a key & all its subkeys by beginning it with '-'.
   integer fw

   fw = open_regfile(regFile)
   if fw = -1 then
      return -1                            -- error
   end if

   for i = 1 to length(extensionList) do
      printf(fw, "[-HKEY_CLASSES_ROOT\\.%s]\n\n", {extensionList[i]})
   end for

   printf(fw, "[-HKEY_CLASSES_ROOT\\%s]", {fileType})

   close_regfile(fw)
   return 0                                -- success
end function

global procedure install (
                          sequence extensionList,
                          sequence_of_byte fileType,
                          sequence_of_byte description,
                          sequence_of_byte iconFile, integer iconNo,
                          sequence_of_byte progFile
                         )
   sequence_of_byte installFile, uninstallFile

   if platform() != WIN32 then
      wait(TITLE &
           "\n\nSorry, -install is only for Windows.")
      abort(1)
   end if

   installFile   = "install.reg"
   uninstallFile = "uninstall.reg"

   if reg_associate(
                    installFile,
                    extensionList,
                    fileType,
                    description,
                    iconFile, iconNo,
                    progFile
                   ) then
      wait(sprintf("Can't open '%s'.", {installFile}))
      abort(2)
   end if

   if reg_unassociate(
                      uninstallFile,
                      extensionList,
                      fileType
                     ) then
      wait(sprintf("Can't open '%s'.", {uninstallFile}))
      abort(2)
   end if

   wait(sprintf("Successfully created files '%s' and '%s'.", {installFile, uninstallFile}))
   abort(0)
end procedure
